import jsonpath
import requests
from urllib.parse import urlparse

from requests import Session


class HttpClient:
    """
    对 requests 库进行简单封装的类，可以方便地进行 HTTP 请求和处理响应。
    """

    def __init__(self, base_url=None):
        """
        初始化方法，接受一个基础 URL 参数，并创建一个 Session 对象。
        :param base_url: str 基础 URL，例如 http://example.com
        """
        if base_url is not None:
            self.base_url = base_url

        self.session: Session = requests.Session()

    def request(self, method, url, **kwargs):
        """
        发送 HTTP 请求，并返回响应对象。
        :param method: str 请求方法，例如 GET、POST、PUT 等。
        :param url: str 请求 URL，会和 base_url 拼接成完整的 URL。
        :param kwargs: dict 其他 requests.request 方法支持的参数。
        :return: requests.Response 响应对象。
        """

        response = self.session.request(method, url, **kwargs)
        # 判断请求url 是否包含 openapi_login  用于处理 单点登录到系统后进行系统的接口请求鉴权处理
        # if 'openapi_login' in url:
        #     # 需要请求接口 获取 Tsessionid
        #     # 先从url 中解析出请求域名
        #     parsed_url = urlparse(url)
        #     domain = parsed_url.scheme + '://' + parsed_url.netloc
        #     # 进行接口请求 拿到响应
        #     personalResponse = self.session.request(method='GET', url=domain + '/api/personal/info/get')
        #     # 拿响应的$.result.authToken
        #     authToken = jsonpath.jsonpath(personalResponse.json(), '$.result.authToken')[0]
        #     # 后续的请求头上都需要带上 Tsessionid
        #     self.session.headers.update({'Tsessionid': authToken})

        http_error_msg = None
        if 400 <= response.status_code < 500:
            http_error_msg = (
                f"{response.status_code} 客户端错误 Error: {response.text} for url: {response.url}"
            )

        elif 500 <= response.status_code < 600:
            http_error_msg = (
                f"{response.status_code} 服务端错误 Error: {response.text} for url: {response.url}"
            );

        # if http_error_msg:
        #     raise HTTPError(http_error_msg, response=response)
        return response

    def get(self, url, params=None, **kwargs):
        """
        发送 GET 请求，并返回响应对象。
        :param url: str 请求 URL，会和 base_url 拼接成完整的 URL。
        :param params: dict 查询参数，会被转换为 ?key=value&key=value 的形式拼接到 URL 后面。
        :param kwargs: dict 其他 requests.get 方法支持的参数。
        :return: requests.Response 响应对象。
        """
        return self.request('GET', url, params=params, **kwargs)

    def post(self, url, json=None, data=None, **kwargs):
        """
        发送 POST 请求，并返回响应对象。
        :param url: str 请求 URL，会和 base_url 拼接成完整的 URL。
        :param data: dict 请求体参数，以表单形式提交。
        :param json: dict 请求体参数，以 JSON 格式提交。
        :param kwargs: dict 其他 requests.post 方法支持的参数。
        :return: requests.Response 响应对象。
        """
        return self.request('POST', url, data=data, json=json, **kwargs)



import logging
import time

"""
日志工具类:
    定义日志输出
"""


class LogConfig:

    @staticmethod
    def logger_set():
        # 第一步：创建一个日志收集器
        log = logging.getLogger()

        # 第二步：设置收集器收集的等级
        log.setLevel(logging.INFO)

        # 第三步：设置输出渠道以及输出渠道的等级
        curTime = time.strftime("%Y-%m-%d %H-%M-%S", time.localtime())
        # fh = logging.FileHandler(f"./log/{curTime}.log", encoding="utf8")
        # fh.setLevel(logging.INFO)
        # log.addHandler(fh)
        # fh.setFormatter(form)

        sh = logging.StreamHandler()
        sh.setLevel(logging.INFO)
        log.addHandler(sh)
        # 创建一个输出格式对象
        formats = '%(thread)d: %(asctime)s -- [%(filename)s-->line:%(lineno)d] - %(levelname)s: %(message)s'
        form = logging.Formatter(formats)
        # 将输出格式添加到输出渠道

        sh.setFormatter(form)

        return log


log = LogConfig.logger_set()



import hmac
import hashlib
import base64
import urllib.parse
import json
from datetime import datetime
import time

import requests

from tinet.ApiConfig import times
from tinet.LogUtil import log



class SignUtil:
    @staticmethod
    def signature(apiUrlPath: str, method, params, access_key_id=None, access_key_secret=None):
        """
         签名算法的封装
        :param apiUrlPath:
        :param method:  RequestMethod.POST/GET
        :param params:  json参数
        :param access_key_id:
        :param access_key_secret:
        :return:
        """
        # 处理url 是否带http
        url = apiUrlPath
        # 处理AK  SK 为空则取环境变量
        access_key_secret = access_key_secret
        access_key_id = access_key_id
        # 获取当前时间戳和签名有效时间
        time1 = datetime.utcfromtimestamp(int(time.time()))
        timestamp = time1.strftime("%Y-%m-%dT%H:%M:%SZ")
        expires = 600
        # 将参数转换为字典，并添加 AccessKeyId、Timestamp 和 Expires 字段
        params_dict_fixed = dict()
        params_dict_fixed['AccessKeyId'] = access_key_id
        params_dict_fixed['Timestamp'] = timestamp
        params_dict_fixed['Expires'] = expires
        params_dict = params_dict_fixed.copy()
        if method.upper() == "GET":
            # 判断参数是否为空
            if params is not None and json.dumps(params).strip():
                params_dict.update(params)
        else:
            params_dict = params_dict
        # 对字典中的参数名进行字典排序
        sorted_params = sorted(params_dict.items(), key=lambda x: x[0])
        # 将参数名和参数值用 & 符号连接，并进行 URL 编码
        encoded_params = urllib.parse.urlencode([(k, v) for k, v in sorted_params])
        # 将请求方法、请求域名和编码后的参数字符串用 ? 符号连接
        # message = str(method) + '\n'+ urllib.parse.urlparse(url).hostname + '\n'+ encoded_params
        message = str(method) + urllib.parse.urlparse(url).hostname + urllib.parse.urlparse(url).path + "?" + encoded_params
        # log.info(f"{url}, 签名前地址: {message}")
        # 使用 AccessKeySecret 密钥对 message 进行哈希计算，并进行 base64 编码
        signature = base64.b64encode(hmac.new(
            access_key_secret.encode(), message.encode(), hashlib.sha1).digest()).decode()
        # 将 signature 进行 URL 编码，并添加到参数字典中
        # 将参数字典转换为字符串，并拼接成完整的请求地址
        if method == 'GET' or method == 'get':
            params_dict['Signature'] = signature
            url_with_signature = url + '?' + urllib.parse.urlencode(params_dict)
        elif method == 'POST' or method == 'post':
            params_dict_fixed['Signature'] = signature
            url_with_signature = url + '?' + urllib.parse.urlencode(params_dict_fixed)
        # log.info(f"{url}, 签名地址: {url_with_signature}")
        return url_with_signature


"""
接口测试核心类
"""
import json
import os
import time
from urllib.parse import urlparse

import pytest
import allure
import jsonpath
from requests_toolbelt import MultipartEncoder
from tinet.LogUtil import log
from tinet.RequestUtil import HttpClient
from tinet.SignUtil import SignUtil
from tinet.YamlUtil import ReadYaml, PlaceholderYaml
from tinet.ApiConfig import config


class APIRequest:
    """
    接口测试核心类
    """

    # 初始化 接收文件 跟一个实体类
    def __init__(self, commonCase=None):
        """
        :param baseUrl: 请求地址
        :param AccessKeyId: AK
        :param AccessKeySecret: SK
        """
        self.commonCase = commonCase
        self.baseUrl = config.baseUrl
        self.AccessKeyId = config.accessKeyId
        self.AccessKeySecret = config.accessKeySecret
        self.globalBean = config
        self.assertFail = config().assertFail
        self.tenv = config.tEnv

    # 请求核心方法, 拿到文件后开始解析
    def doRequest(self, file, bean):
        # 请求的时候最终参数  先替换parameter中的占位符  再判断鉴权方式,如果是SIGN的POST就完整parameter否则GET就为空
        requestParameter = None
        dataSaveBean = bean
        yaml = ReadYaml(file).load_yaml()
        yamlId = yaml.get('id')
        yamlName = yaml.get('name')
        yamlKind = yaml.get('kind')
        yamlTestcase = yaml.get('testcases')
        # 先拿到所有testcase
        log.info(f"开始执行测试用例name: {yamlName}, id: {yamlId} , 测试用例: {yamlTestcase}")
        allure.attach(f"{yamlTestcase}", name=f"开始执行测试用例name: {yamlName}, id: {yamlId} ", attachment_type=allure.attachment_type.TEXT)
        # 添加公共session  判断是否有公共session
        clientSession = config.Session
        # 开始遍历testcase
        # allure.dynamic.title(yamlName)
        for testcase in yamlTestcase:
            with allure.step(testcase.get('name')):
                # 拿到用例如果是common的去common文件路径中拿测试用例
                if testcase.get('skip'):  # 判断是否跳过
                    log.info(f"用例: {json.dumps(testcase.get('name'), indent=4, ensure_ascii=False)}跳过")
                    continue
                sleeps = testcase.get('sleep')
                if sleeps:  # 判断是否需要等待
                    time.sleep(int(sleeps))
                    log.info(f"当前用例: {json.dumps(testcase.get('name'), indent=4, ensure_ascii=False)}执行前等待{sleeps}秒")
                if testcase.get('kind') and testcase.get('kind').lower() == 'common' and config.commonTestCasePath is not None:
                    log.info("读取common文件中的测试用例,匹配测试用例id: {}".format(testcase.get('id')))
                    allure.attach(f"读取common文件中的测试用例,匹配测试用例id: {testcase.get('id')}", name=f"读取common文件中的测试用例,匹配测试用例id: {testcase.get('id')}",
                                  attachment_type=allure.attachment_type.TEXT)
                    # 拿到common文件中的测试用例 然后重新赋值给testcase
                    testcase = self.getCommonTestCase(testcase, config.commonTestCasePath, testcase.get('id'))
                elif testcase.get('kind') and testcase.get('kind').lower() == 'common' and config.commonTestCasePath is None:
                    log.error(f"commonPath路径未配置,请检查配置文件")
                    raise Exception(f"commonPath路径未配置,请检查配置文件")
                # 执行当前用例 格式化为json 间隔4个空格
                log.info(f"执行当前用例name: {testcase.get('name')}, id: {testcase.get('id')}")
                allure.attach(f"{json.dumps(testcase, indent=4, ensure_ascii=False)}", name=f"执行当前用例name: {testcase.get('name')}, id: {testcase.get('id')}",
                              attachment_type=allure.attachment_type.TEXT)
                if self.tenv == 'gray':
                    testcase.get('headers')['t-env'] = 'gray'
                # 1. 先看看参数中是否有需要替换的参数
                if testcase.get('requestType') is None:
                    requestType = 'json'
                else:
                    requestType = testcase.get('requestType')
                repParameter = self.replaceParameterAttr(dataSaveBean, testcase.get('parameter'), requestType)
                repApi = self.replaceParameterAttr(dataSaveBean, testcase.get('api'))
                headers = self.replaceParameterAttr(dataSaveBean, testcase.get('headers'))
                # 2. 判断鉴权方式 authType 返回请求地址
                requestParameter, requestUrl = self.authType(testcase.get('authType'), repApi, testcase.get('method'), repParameter)
                # 3. 判断请求 requestType
                dataRequestParameter, jsonRequestParameter, paramsData, ModelData, requestType = self.requestType(requestType, requestParameter)
                # 4. 开始请求
                log.info(f"开始请求地址: {requestUrl} ")
                allure.attach(f"{requestUrl}", name=f"开始请求地址", attachment_type=allure.attachment_type.TEXT)
                log.info(f"开始请求方式: {testcase.get('method')}")
                allure.attach(f"{testcase.get('method')}", name=f"开始请求方式", attachment_type=allure.attachment_type.TEXT)
                if dataRequestParameter is not None:
                    log.info(f"开始请求参数dataRequestParameter: {dataRequestParameter}")
                    allure.attach(f"{dataRequestParameter}", name=f"开始请求参数dataRequestParameter", attachment_type=allure.attachment_type.TEXT)
                if jsonRequestParameter is not None:
                    log.info(f"开始请求参数jsonRequestParameter: {jsonRequestParameter}")
                    allure.attach(f"{jsonRequestParameter}", name=f"开始请求参数jsonRequestParameter", attachment_type=allure.attachment_type.TEXT)
                if paramsData is not None:
                    log.info(f"开始请求参数paramsData: {paramsData}")
                    allure.attach(f"{paramsData}", name=f"开始请求参数paramsDatar", attachment_type=allure.attachment_type.TEXT)
                if ModelData is not None:
                    log.info(f"开始请求参数ModelData: {ModelData}")
                    allure.attach(f"{ModelData}", name=f"开始请求参数ModelData", attachment_type=allure.attachment_type.TEXT)
                # form-data请求 需要处理一下header
                if dataRequestParameter is not None and requestType.lower() == 'form-data' or requestType.lower() == 'form-file':
                    headers['Content-Type'] = dataRequestParameter.content_type
                response = clientSession.request(method=testcase.get('method'), url=requestUrl, data=dataRequestParameter, json=jsonRequestParameter, params=paramsData, files= ModelData, headers=headers)
                try:  # 当返回不是 json 返回text
                    log.info(f"当前用例response: {json.dumps(response.json(), indent=4, ensure_ascii=False)}")
                    allure.attach(f"{json.dumps(response.json(), indent=4, ensure_ascii=False)}", name=f"当前用例response",
                                  attachment_type=allure.attachment_type.TEXT)
                except:
                    log.info(f"当前用例response: {json.dumps(response.text)}")
                    allure.attach(f"{json.dumps(response.text)}", name=f"当前用例response",
                                  attachment_type=allure.attachment_type.TEXT)
                # 5. 处理断言
                if testcase.get('assertFail'):
                    failtype = testcase.get('assertFail')
                else:
                    failtype = self.assertFail
                self.assertType(testcase.get('assert'), response, dataSaveBean, failtype)
                # 6. 处理response,根据data 来进行变量存取
                # try 里面的代码是为了处理response不是json格式的情况
                try:
                    self.addAttrSaveBean(dataSaveBean, self.globalBean, testcase.get('saveData'), response.json())
                except:
                    self.addAttrSaveBean(dataSaveBean, self.globalBean, testcase.get('saveData'), response.text)
        return clientSession

    # 处理断言方法  - eq: [ 'body.userId', 1 ]  通过testcase中的assert来判断断言方式  先取断言的方式 再取断言的内容跟值
    def assertType(self, assertType, response, bean, failType):
        log.info(f"开始处理断言: {assertType}")
        allure.attach(f"{assertType}", name=f"开始处理断言", attachment_type=allure.attachment_type.TEXT)
        # 判断assertType是否为空
        if assertType is None:
            log.info(f"断言为空,跳过断言")
            allure.attach(f"断言为空,跳过断言", name=f"断言为空,跳过断言", attachment_type=allure.attachment_type.TEXT)
            return None
        # 开始判断断言方式
        # [{'eq': ['body.userId', 1]}, {'eq': ['body.userId', 1]}] 循环判断
        for ass in assertType:
            key = list(ass.keys())[0]
            log.info(f"开始判断{key}断言: {ass.get(key)}")
            allure.attach(f"{ass.get(key)}", name=f"开始判断{key}断言", attachment_type=allure.attachment_type.TEXT)
            if 'status_code' in ass:
                # 断言 响应状态码 status_code
                # 取要断言的值
                if ass.get('status_code'):
                    log.info(f"断言status_code结束,{ass.get('status_code')}  ,response结果: {response.status_code}")
                    allure.attach(f"{ass.get('status_code')}  ,response结果: {response.status_code}", name=f"断言status_code结束", attachment_type=allure.attachment_type.TEXT)
                    self.assertChoose(str(response.status_code) == str(ass.get('status_code')), f"status_code断言失败: {ass.get('status_code')}  ,response结果: {response.status_code}", failType)
                    continue
            jsonpathResults = jsonpath.jsonpath(response.json(), ass.get(key)[0])
            if jsonpathResults is False:
                log.info(f'提取{ass.get(key)[0]}失败，请检查格式')
                allure.attach(f"提取{ass.get(key)[0]}失败，请检查格式", name=f"提取{ass.get(key)[0]}失败，请检查格式", attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(1 > 2, f"提取{ass.get(key)[0]}失败，断言失败, response结果: {json.dumps(response.text, indent=4, ensure_ascii=False)}", failType)
                continue
            if 'eq' in ass:
                # 相等判断 都转为str进行判断
                expectedResults = PlaceholderYaml(attrObj=bean, reString=ass.get('eq')[1]).replace().replaced_str
                # 判断 expectedResults 是否在 jsonpathResults list中  判断类型
                assResults = str(expectedResults) in [str(item) for item in jsonpathResults]
                log.info(f"断言eq结束: {jsonpathResults} 等于 {expectedResults}")
                allure.attach(f"{jsonpathResults} 等于 {expectedResults}", name=f"断言eq结束", attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(assResults is True, f"eq断言失败: {jsonpathResults} 不等于 {expectedResults}", failType)
            elif 'sge' in ass:
                # size greater than or equal
                expectedResults = PlaceholderYaml(attrObj=bean, reString=ass.get('sge')[1]).replace().replaced_str
                log.info(f"断言sge结束: {jsonpathResults} size >= {expectedResults}")
                allure.attach(f" {jsonpathResults} size >= {expectedResults}", name=f"断言sge结束",
                              attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(len(jsonpathResults) >= int(expectedResults), f"sge断言失败: {jsonpathResults} 小于 {expectedResults}", failType)
            elif 'ne' in ass:
                # not eq 断言
                return 'ne'
            elif 'nn' in ass:
                # not none
                # 判断json path 是否不为空 返回FALSE
                log.info(f"断言not none结束: {jsonpathResults}")
                allure.attach(f" {jsonpathResults} ", name=f"断言not none结束", attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(jsonpathResults is not None,
                                  f"not none断言失败: {ass.get('nn')[0]}  ,jsonpath结果: {jsonpathResults}", failType)
            elif 'none' in ass:
                # 判断json path 是否 返回FALSE
                log.info(f"断言none结束: {jsonpathResults}")
                allure.attach(f" {jsonpathResults} ", name=f"断言none结束", attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(jsonpathResults is True,
                                  f"none断言失败: {ass.get('none')[0]}  ,jsonpath结果: {jsonpathResults}", failType)
            elif 'in' in ass:
                # 判断预期值是否被返回值包含
                expectedResults = PlaceholderYaml(attrObj=bean, reString=ass.get('in')[1]).replace().replaced_str
                log.info(f"断言in结束: {expectedResults} 在 {jsonpathResults} 内")
                allure.attach(f" {expectedResults} 在 {jsonpathResults} 内", name=f"断言in结束",
                              attachment_type=allure.attachment_type.TEXT)
                self.assertChoose(str(expectedResults) in str(jsonpathResults), f"断言in失败: {expectedResults} 不在 {jsonpathResults} 内", failType)

    # 根据data从response中根据json还是其他方法来获取response中的值
    def addAttrSaveBean(self, bean, globalBean, data: list, response):
        # 先解析data 如果data为空则直接返回
        if data is None:
            return
        for d in data:
            log.info(f"获取要添加的属性表达式: data: {d}")
            allure.attach(f"{d}", name=f"获取要添加的属性表达式: data", attachment_type=allure.attachment_type.TEXT)
            # 判断 d 的key 是否为json 怎走jsonpath 解析
            if 'json' in d:
                log.info(f"开始解析json: {d.get('json')}")
                allure.attach(f"{d.get('json')}", name=f"开始解析json", attachment_type=allure.attachment_type.TEXT)
                # 解析json
                jsonPath = d.get('json')[1]
                # 获取jsonpath的值
                value = jsonpath.jsonpath(response, jsonPath)
                # 判断value是为False
                if value is False:
                    value = None
                log.info(f"解析完成jsonPath: {jsonPath} 的值: {value}")
                allure.attach(f"{jsonPath} 的值: {value}", name=f"解析完成jsonPath", attachment_type=allure.attachment_type.TEXT)
                # 给bean添加属性
                # 判断value是否为空 并且 value 不是false 并且value的长度是否为1 如果是则直接赋值 如果不是则赋值list
                # 如果value为空则不赋值 是多个则 赋值list
                saveBean = bean
                if d.get('json').__len__() == 3 and d.get('json')[2].lower() == 'global':
                    saveBean = globalBean
                else:
                    saveBean = bean
                if value is not None and len(value) > 1:
                    setattr(saveBean, d.get('json')[0], list(value))
                    log.info(f"添加属性完成: {d.get('json')[0]} = {list(value)}")
                    allure.attach(f"{d.get('json')[0]} = {list(value)}", name=f"添加属性完成", attachment_type=allure.attachment_type.TEXT)
                elif value is not None and len(value) == 1:
                    setattr(saveBean, d.get('json')[0], value[0])
                    log.info(f"添加属性完成: {d.get('json')[0]} = {value[0]}")
                    allure.attach(f"{d.get('json')[0]} = {value[0]}", name=f"添加属性完成", attachment_type=allure.attachment_type.TEXT)
                else:
                    log.error(f"添加属性失败: {d.get('json')[0]} = {value}")
                    allure.attach(f"{d.get('json')[0]} = {value}", name=f"添加属性失败", attachment_type=allure.attachment_type.TEXT)
            elif 'other' in d:
                log.info(f"开始解析other: {d.get('other')}")
                allure.attach(f"{d.get('other')}", name=f"开始解析other", attachment_type=allure.attachment_type.TEXT)

    # 拿到参数后先替换变量或者方法
    def replaceParameterAttr(self, bean, parameter, requestType='json'):
        # 先拿到所有的参数
        log.info(f"开始替换参数: {parameter}")
        allure.attach(f"{parameter}", name=f"开始替换参数", attachment_type=allure.attachment_type.TEXT)
        # 判断参数是否为空
        if parameter is None:
            return None
        # 开始调用方法替换
        if requestType.lower() == 'json-text':
            repParameter = PlaceholderYaml(yaml_str=parameter, attrObj=bean).replace().textLoad()
        else:
            repParameter = PlaceholderYaml(yaml_str=parameter, attrObj=bean).replace().jsonLoad()
        log.info(f"替换后的参数: {repParameter}")
        allure.attach(f"{repParameter}", name=f"替换后的参数", attachment_type=allure.attachment_type.TEXT)
        return repParameter

    # 判断请求方式 requestType
    def requestType(self, requestType, data):
        # 判断是否为空
        jsonRequestParameter = None
        dataRequestParameter = None
        paramsData = None
        ModelData = None
        if requestType is None:
            log.info(f"请求方式为空: {requestType} ,默认走json请求")
            allure.attach(f"{requestType}", name=f"请求方式为空,默认走json请求", attachment_type=allure.attachment_type.TEXT)
            jsonRequestParameter = data
        elif requestType.lower() == "json" or requestType.lower() == "json-text":
            # 进行json请求 JSON-TEXT: 入参含有str类型的字典使用
            log.info(f"请求方式为JSON: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为JSON", attachment_type=allure.attachment_type.TEXT)
            jsonRequestParameter = data
        elif requestType.lower() == "form-data":
            # 进行form请求, 该类型 不能传文件
            log.info(f"请求方式为FORM-DATA: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为FORM", attachment_type=allure.attachment_type.TEXT)
            dataRequestParameter = MultipartEncoder(fields=data)
        elif requestType.lower() == "form-model":
            # 进行form-model请求，该类型可以传文件，格式为 form-data
            log.info(f"请求方式为FORM-MODEL: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为FORM-Model", attachment_type=allure.attachment_type.TEXT)
            filename = data['filename']  # 获取文件字段
            log.info(type(data))
            log.info(data[filename])
            file_name = data[filename].split('\\')
            log.info(file_name[-1])
            data[filename] =  (file_name[-1], open(data[filename], 'rb'), 'application/octet-stream')
            for k,v in data.items():
                if type(v) == dict:
                    data[k] = (None, json.dumps(data[k]))
            log.info(data)
            ModelData = data
        elif requestType.lower() == "form-file":
            # 进行form-file请求，该类型可以传文件,格式为json
            log.info(f"请求方式为FORM-FILE: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为FORM-FILE", attachment_type=allure.attachment_type.TEXT)
            filename = data['filename']  # 获取文件字段
            data[filename] = (os.path.basename(data[filename]), open(data[filename], 'rb'), 'application/octet-stream')
            dataRequestParameter = MultipartEncoder(fields=data)
        elif requestType == "PARAMS":
            # 进行PARAMS请求
            log.info(f"请求方式为PARAMS: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为PARAMS", attachment_type=allure.attachment_type.TEXT)
            paramsData = data
        elif requestType == "DATA":
            # 进行DATA请求
            log.info(f"请求方式为DATA: {requestType}")
            allure.attach(f"{requestType}", name=f"请求方式为DATA", attachment_type=allure.attachment_type.TEXT)
            dataRequestParameter = data
        else:
            log.error("请求方式不支持")
            allure.attach(f"请求方式不支持", name=f"请求方式不支持", attachment_type=allure.attachment_type.TEXT)
        return dataRequestParameter, jsonRequestParameter, paramsData, ModelData, requestType

    # 判断鉴权方式 authType
    def authType(self, authType, url, method, parameter):
        # 判断baseurl 是否为None
        if self.baseUrl is None or self.isValidUrl(url):
            requestUrl = url
        else:
            requestUrl = self.baseUrl + url
        requestParameter = None
        if authType == "SIGN":
            # 进行签名
            requestUrl = SignUtil.signature(apiUrlPath=requestUrl, method=method, params=parameter, access_key_id=self.AccessKeyId, access_key_secret=self.AccessKeySecret)
            if method == "GET":
                requestParameter = None
            elif method == "POST":
                requestParameter = parameter
            allure.attach(f"{requestUrl}", name=f"SIGN鉴权请求", attachment_type=allure.attachment_type.TEXT)
            return requestParameter, requestUrl
        elif authType == "COOKIE" or authType is None:
            # 进行cookie
            requestParameter = parameter
            allure.attach(f"{requestUrl}", name=f"COOKIE鉴权请求", attachment_type=allure.attachment_type.TEXT)
            return requestParameter, requestUrl
        elif authType == "AUTH":
            # 进行auth
            pass
        else:
            log.error("鉴权方式不支持")
        return requestParameter, requestUrl

    # 根据common路径跟id 去重新赋值testcase
    def getCommonTestCase(self, testcase, commonFile, caseId):
        # 先判断 self.commonCase 是否为空,如果不为空则需要读取commonFile,否则读取用例类自定义commonfile
        if self.commonCase is None:
            # 读取commonFile
            commonFile = commonFile
        else:
            commonFile = os.path.join(commonFile.split('common')[0], f'common/{self.commonCase}')
        log.info(f"开始读取commonFile: {commonFile}")
        allure.attach(f"{commonFile}", name=f"开始读取commonFile", attachment_type=allure.attachment_type.TEXT)
        yaml = ReadYaml(commonFile).load_yaml()
        # 获取所有case
        commonCase = yaml.get('testcases')
        log.info(f"读取完成commonFile: {commonCase}")
        allure.attach(f"{commonCase}", name=f"读取完成commonFile", attachment_type=allure.attachment_type.TEXT)

        # 遍历commonCase ,如果id相同则返回case 如果遍历完没有找到则报错
        for case in commonCase:
            if case.get('id') == caseId:
                log.info(f"找到commonCase: {case}")
                allure.attach(f"{case}", name=f"找到commonCase", attachment_type=allure.attachment_type.TEXT)
                # 如果找到则返回case 将case的assert与testcase的assert进行合并
                case['assert'] = [item for item in (case.get('assert') or []) + (testcase.get('assert') or []) if
                                  item is not None]
                case['saveData'] = [item for item in (case.get('saveData') or []) + (testcase.get('saveData') or []) if
                                    item is not None]
                return case
        log.error(f"没有找到commonCase: {caseId}")
        allure.attach(f"{caseId}", name=f"没有找到commonCase", attachment_type=allure.attachment_type.TEXT)
        raise ValueError("Case with id {} not found".format(caseId))

    def assertChoose(self, ass, tips, type):
        """
        assertChoose 断言选择器
        :param ass:
        :param tips:
        :param type:
        :return:
        """
        if type == 'stop':
            assert ass, tips
        elif type == 'continue':
            pytest.assume(ass, tips)

    def isValidUrl(self, url):
        try:
            result = urlparse(url)
            return all([result.scheme, result.netloc])
        except ValueError:
            return False



from faker import Faker
import string, time
import random
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
import datetime
import pytz
import hashlib
import base64

from tinet.RequestUtil import HttpClient


class times:
    def __init__(self, dt=None, tz=pytz.timezone('Asia/Shanghai')):
        """
        初始化时间工具类

        :param dt: datetime对象，默认使用当前时间
        :param tz: 时区，默认使用本地时区
        """
        self.tz = tz
        self.dt = dt or datetime.datetime.now(self.tz)

    def to_datetime(self):
        """
        返回datetime对象

        :return: datetime对象
        """
        return self.dt

    def to_str(self, fmt='%Y-%m-%dT%H:%M:%SZ'):
        """
        将datetime对象转换为指定格式的字符串

        :param fmt: 时间格式，默认为'%Y-%m-%d %H:%M:%S'
        :return: 格式化后的时间字符串
        """
        return self.dt.strftime(fmt)


class ChineseDataGenerator:
    """
    生成中文数据的工具类
    """

    def __init__(self):
        self.fake = Faker(locale='zh_CN')

    def generate_name(self):
        """
        生成中文姓名，返回字符串。
        """
        return self.fake.name()

    def generate_address(self):
        """
        生成中文地址，返回字符串。
        """
        return self.fake.address()

    def generate_phone_number(self):
        """
        生成中文手机号，返回字符串。
        """
        return self.fake.phone_number()

    def generate_id_number(self):
        """
        生成中文身份证号码，返回字符串。
        """
        return self.fake.ssn()

    def random_number(self, digits=4):
        """
        生成一个4位随机整数并转换为字符串类型
        如果生成的整数不足4位，则在左侧用0进行填充
        """
        digits = int(digits)
        return "{:04d}".format(self.fake.random_number(digits=digits))

    def get_sk_password(self):
        """
        获取初始化的SK 加密后的密码
        :return:
        """
        return config.SKPassword

    @staticmethod
    def start_of_day():
        """
        获取当前时间开始时间戳：eg：2023-06-01 00:00:00
        """
        now = datetime.datetime.now()
        return datetime.datetime(now.year, now.month, now.day)

    @staticmethod
    def end_of_day():
        """
        获取当前时间开始时间戳：eg：2023-06-01 23:59:59
        """
        return ChineseDataGenerator.start_of_day() + datetime.timedelta(days=1) - datetime.timedelta(seconds=1)

    def start_of_day_s(self):
        """
        获取当前时间开始时间戳：eg：1685548800  秒级
        """
        return int(time.mktime(ChineseDataGenerator.start_of_day().timetuple()))

    def end_of_day_s(self):
        """
        获取当前时间结束时间戳：eg：1685635199 秒级
        """
        return int(time.mktime(ChineseDataGenerator.end_of_day().timetuple()))

    def random_externalId(self):
        """
        生成唯一性数据，crm用于 外部企业客户id
        """
        num = str(random.randint(1000, 9999))
        src_uppercase = string.ascii_uppercase  # string_大写字母
        src_lowercase = string.ascii_lowercase  # string_小写字母
        chrs = random.sample(src_lowercase + src_uppercase, 3)
        for i in chrs:
            num += i
        return num

    def encryptPassword(self, plain_text='Aa112233'):
        """
        # 加密
        """
        password = config.accessKeySecret
        # 设置随机数生成器的种子
        secure_random = hashlib.sha1(password.encode()).digest()
        # 创建对称加密密钥生成器
        kgen = hashlib.sha1(secure_random).digest()[:16]
        # 创建密码器并初始化
        cipher = AES.new(kgen, AES.MODE_ECB)
        # 加密明文（使用PKCS7填充）
        padded_plain_text = pad(plain_text.encode(), AES.block_size)
        encrypted_bytes = cipher.encrypt(padded_plain_text)
        # 将加密结果转换为16进制字符串
        encrypted_text = base64.b16encode(encrypted_bytes).decode().lower()
        return encrypted_text

    def decrypt(self, encrypted_text, password):
        """
        # 解密
        """
        # 设置随机数生成器的种子
        secure_random = hashlib.sha1(password.encode()).digest()
        # 创建对称加密密钥生成器
        kgen = hashlib.sha1(secure_random).digest()[:16]
        # 创建密码器并初始化
        cipher = AES.new(kgen, AES.MODE_ECB)
        # 解密密文（parseHexStr2Byte方法为将16进制字符串转为二进制字节数组）
        encrypted_bytes = base64.b16decode(encrypted_text)
        decrypted_bytes = cipher.decrypt(encrypted_bytes)
        decrypted_text = unpad(decrypted_bytes, AES.block_size).decode()
        return decrypted_text


class config:
    """
    配置文件
    """

    def __init__(self, baseUrl=None, AccessKeyId=None, AccessKeySecret=None, SKPassword=None, commonTestCasePath=None, methObj=None,
                 Session: HttpClient = HttpClient(), assertFail='stop', tEnv='base'):
        """
        初始化配置文件
        """
        self._baseUrl = baseUrl
        self._accessKeyId = AccessKeyId
        self._accessKeySecret = AccessKeySecret
        # SK加密 登录密码 后的 密码加密文本
        self._SKPassword = SKPassword
        self._commonTestCasePath = commonTestCasePath
        self._methObj = methObj
        self._assertFail = assertFail
        self._tEnv = tEnv
        # 构建全局session
        self._Session = Session

    @property
    def Session(self):
        return self._Session

    @Session.setter
    def Session(self, value):
        self._Session = value

    @property
    def methObj(self):
        return self._methObj

    @methObj.setter
    def methObj(self, value):
        self._methObj = value

    @property
    def SKPassword(self):
        return self._SKPassword

    @SKPassword.setter
    def SKPassword(self, value):
        self._SKPassword = value

    @property
    def commonTestCasePath(self):
        return self._commonTestCasePath

    @commonTestCasePath.setter
    def commonTestCasePath(self, value):
        self._commonTestCasePath = value

    @property
    def baseUrl(self):
        return self._baseUrl

    @baseUrl.setter
    def baseUrl(self, value):
        self._baseUrl = value

    @property
    def accessKeyId(self):
        return self._accessKeyId

    @accessKeyId.setter
    def accessKeyId(self, value):
        self._accessKeyId = value

    @property
    def accessKeySecret(self):
        return self._accessKeySecret

    @accessKeySecret.setter
    def accessKeySecret(self, value):
        self._accessKeySecret = value

    @property
    def assertFail(self):
        return self._assertFail

    @assertFail.setter
    def assertFail(self, value):
        self._assertFail = value

    @property
    def tEnv(self):
        return self._tEnv

    @tEnv.setter
    def tEnv(self, value):
        self._tEnv = value



from tinet.APIRequest import APIRequest
from tinet.ApiConfig import config, ChineseDataGenerator
from tinet.RequestUtil import HttpClient

apiConfig = config()
config.methObj = ChineseDataGenerator()
config.Session = HttpClient()



import re
import yaml
import json
from tinet.LogUtil import log
from tinet.ApiConfig import config


class PlaceholderYaml:
    """
    用于替换 yaml 文件中的占位符
    """

    def __init__(self, yaml_str=None, reString=None, attrObj=config, methObj=None, gloObj=config):
        if yaml_str:
            self.yaml_str = json.dumps(yaml_str)
        else:
            self.yaml_str = str(reString)

        self.attrObj = attrObj
        self.methObj = config.methObj if methObj is None else methObj
        self.gloObj = gloObj

    def replace(self):
        # 定义正则表达式模式
        # 用于匹配 ${attr} 和 #{method} 这样的占位符
        # $() #() 如果不匹配则报错
        pattern_attr = re.compile(r'\$\{(\w+)\}')
        pattern_method = re.compile(r'\#\{(.*?)\}')
        pattern_glo = re.compile(r'\$\$\{(\w+)\}')

        # 定义全局替换函数
        def replace_glo(match):
            # 获取占位符中的属性名
            attr_name = match.group(1)
            # 如果对象中有该属性，则返回该属性的值
            if hasattr(self.gloObj, attr_name):
                # 获取属性的值
                attr_value = getattr(self.gloObj, attr_name)
                # 如果属性的值是字符串，则返回该字符串
                if isinstance(attr_value, str):
                    return str(attr_value)
                # 如果属性的值是可调用对象，则返回方法名
                elif callable(attr_value):
                    return match.group(0)
                # 如果属性的值是字典，则返回该字典的字符串表示
                elif isinstance(attr_value, dict):
                    return str(attr_value)
                # 如果属性的值是列表，则返回该列表的字符串表示
                elif isinstance(attr_value, list):
                    return str(",".join(str(x) for x in attr_value))
                # 否则，返回属性的值（将其转换为字符串）
                else:
                    return str(attr_value)
            # 否则返回原字符串
            return match.group(0)

        # 定义替换函数
        def replace_attr(match):
            # 获取占位符中的属性名
            attr_name = match.group(1)
            # 如果对象中有该属性，则返回该属性的值
            if hasattr(self.attrObj, attr_name):
                # 获取属性的值
                attr_value = getattr(self.attrObj, attr_name)
                # 如果属性的值是字符串，则返回该字符串
                if isinstance(attr_value, str):
                    return str(attr_value)
                # 如果属性的值是可调用对象，则返回方法名
                elif callable(attr_value):
                    return match.group(0)
                # 如果属性的值是字典，则返回该字典的字符串表示
                elif isinstance(attr_value, dict):
                    return str(attr_value)
                # 如果属性的值是列表，则返回该列表的字符串表示
                elif isinstance(attr_value, list):
                    return str(",".join(str(x) for x in attr_value))
                # 否则，返回属性的值（将其转换为字符串）
                else:
                    return str(attr_value)
            # 否则返回原字符串
            return match.group(0)

        # 定义替换函数
        def replace_method(match):
            # 获取占位符中的方法名
            method_name = match.group(1)
            args = None
            if '(' in match.group(1):
                # 获取占位符中的方法名
                method_name = match.group(1).split('(')[0]
                # 获取参数列表
                args_str = match.group(1).split('(')[1][:-1]
                args = [arg.strip() for arg in args_str.split(',')]

            # 如果对象中有该方法，并且该方法是可调用的，则返回该方法的返回值
            if hasattr(self.methObj, method_name):
                # 获取方法
                method = getattr(self.methObj, method_name)
                # 如果方法是可调用对象，则调用该方法并返回其返回值的字符串表示
                if callable(method):
                    if args:
                        method_value = method(*args)
                    else:
                        method_value = method()
                    if isinstance(method_value, str):
                        return str(method_value)
                    else:
                        return str(method_value)
                # 否则，返回方法的字符串表示
                else:
                    return str(method)
            # 否则返回原字符串
            return match.group(0)

        # todo 判断是否有需要替换的 再进行替换
        log.info(f"开始替换str中的占位符: {self.yaml_str}")
        # 先进行全局替换
        replaced_str = pattern_glo.sub(replace_glo, self.yaml_str)
        # 替换占位符中的属性
        replaced_str = pattern_attr.sub(replace_attr, replaced_str)
        # 替换占位符中的方法
        replaced_str = pattern_method.sub(replace_method, replaced_str)
        self.replaced_str = replaced_str
        log.info("替换后的str内容为：{}".format(replaced_str))
        return self

    def jsonLoad(self):
        # 把replaced_str中的"[]"替换为[]  "{}"替换为{}   "'"替换为"/""  'None换为 null'
        replaced_str = self.replaced_str.replace('"[', '[').replace(']"', ']').replace('"{', '{').replace('}"', '}').replace("'", "\"").replace('None','null')
        try:
            replaced_str = json.loads(replaced_str)
            log.info("替换后jsonLoad的str内容为：{}".format(replaced_str))
            return replaced_str
        except:
            log.info(f'*************替换失败-YAML,请检查格式{replaced_str}******************')

    def textLoad(self):
        return json.loads(self.replaced_str)


class ReadYaml:
    """
    用于读取 yaml 文件的工具类
    """

    def __init__(self, yaml_file):
        self.yaml_file = yaml_file

    def load_yaml(self):
        """
        读取 yaml 文件，并返回其中的数据。
        :return: dict
        """

        with open(self.yaml_file, encoding='utf-8') as f:
            data = yaml.safe_load(f)
        return data if data is not None else {}  # 如果 data 为 None，则返回空字典

    def get(self, key, default=None):
        """
        获取 yaml 文件中的数据
        :param key: 数据的键
        :param default: 如果获取失败，则返回该默认值
        :return: dict
        """

        # 读取 yaml 文件
        data = self.load_yaml()
        # 获取数据
        return data.get(key, default) if data is not None else default

    def get_all(self):
        """
        获取 yaml 文件中的所有数据
        :return: dict
        """

        # 读取 yaml 文件
        data = self.load_yaml()
        # 如果 data 为 None，则返回空字典
        return data if data is not None else {}



from pathlib import Path
from setuptools import find_namespace_packages, setup

# Load packages from requirements.txt
BASE_DIR = Path(__file__).parent
with open(Path(BASE_DIR, "requirements.txt"), "r") as file:
    required_packages = [ln.strip() for ln in file.readlines()]

# Define our package
setup(
    name="TinetApiRequest",
    version="1.2.4",
    description="天润接口测试库",
    author="天润-测试",
    author_email="zhupeng@ti-net.com.cn",
    url="https://www.ti-net.com.cn/",
    python_requires=">=3.7",
    packages=find_namespace_packages(),
    install_requires=[required_packages],
    license='MIT',
)
# python setup.py sdist bdist_wheel
# twine upload dist/*
# zhupengtinet
# 账号: __token__
# 密码: pypi-AgEIcHlwaS5vcmcCJDE5ZGVjOTc0LWY1YjctNGVhMi05OWFlLWZhNzFhYWI1MDcyOQACKlszLCI0NTIzNzJkYi0xNzEyLTQ2YjUtYTBmZi1jYTM0ZTQxNGViMjIiXQAABiD0abj-vqlyLS22IMYQiVOPkRtCk19oCyRdMuUXS2ahiw
# 账号 zp
# pypi-AgEIcHlwaS5vcmcCJGU3NThhZGNiLTlhZmItNGQ4OC1hNmRiLWMzNTgwMDM0ZGZjNgACKlszLCI0NTIzNzJkYi0xNzEyLTQ2YjUtYTBmZi1jYTM0ZTQxNGViMjIiXQAABiA-VFBYRmrG-lcYgoo-rR2zXIfYqSRyNlYY6GxMm6Vhhw
# [pypi]
#   username = __token__
#   password = pypi-AgEIcHlwaS5vcmcCJGZiNGZiMjVlLTk0YTgtNGEyZS1iZTQ3LWQ2MjhmZmVlNTg3ZAACKlszLCI0NTIzNzJkYi0xNzEyLTQ2YjUtYTBmZi1jYTM0ZTQxNGViMjIiXQAABiDKrO1hnxV3RBZvzZhGREgDPkZfEut6DnsAhdjWmZnIvA
#   pypi-AgEIcHlwaS5vcmcCJDE5ZGVjOTc0LWY1YjctNGVhMi05OWFlLWZhNzFhYWI1MDcyOQACKlszLCI0NTIzNzJkYi0xNzEyLTQ2YjUtYTBmZi1jYTM0ZTQxNGViMjIiXQAABiD0abj-vqlyLS22IMYQiVOPkRtCk19oCyRdMuUXS2ahiw
# safeCode
# PyPI recovery codes
# 7dc78210b25a85ce
# 354fd734f3cec93c
# 053c971d923a28e0
# 62ad3ac15c089920
# 78ec32d80ce37a9c
# 05ca7615d3a2192f
# 117c2bab077b6cd0
# 8840385f2976feb9
